local DefileCalendar = {
  frame = CreateFrame("Frame"),
  set = 0  
}

local CALENDAR_MONTH_NAMES = {
	MONTH_JANUARY,
	MONTH_FEBRUARY,
	MONTH_MARCH,
	MONTH_APRIL,
	MONTH_MAY,
	MONTH_JUNE,
	MONTH_JULY,
	MONTH_AUGUST,
	MONTH_SEPTEMBER,
	MONTH_OCTOBER,
	MONTH_NOVEMBER,
	MONTH_DECEMBER,
};

DefileCalendar_EventRowArray = {}   
DefileCalendar_EventHeaderArray = {}              

defileCaldendarFrame_Updatehooksw = false

DefileCalendar.frame:SetScript("OnEvent", function(self, event, ...)
	if event == "PLAYER_ENTERING_WORLD" and DefileCalendar.set == 0 then
		DefileCalendar.set = 1
		if not defileCaldendarFrame_Updatehooksw then
			ToggleCalendar()
			ToggleCalendar()
			CalendarFrame:HookScript("OnShow", showDefileCalendar)
			CalendarFrame:HookScript("OnHide", hideDefileCalendar)
			defileCaldendarFrame_Updatehooksw = true
		end
	end

	if event == "CALENDAR_UPDATE_EVENT_LIST" then
		loadDefileEvents()
	end
end)
      
local function _GetDayOfWeek(index)
	return mod(index - 1, 7) + 1;
end

local function _GetWeekdayIndex(index)
	-- the expanded form for the left input to mod() is:
	-- (index - 1) + (CALENDAR_FIRST_WEEKDAY - 1)
	-- why the - 1 and then + 1 before return? because lua has 1-based indexes! awesome!
	return mod(index - 2 + CALENDAR_FIRST_WEEKDAY, 7);
end
	  
function loadDefileEvents()
	local frameChildren = { DefileCalendar_Main_ScrollFrame:GetChildren() }
	for frameChild = #frameChildren,1,-1 do -- _, frameChild in ipairs(frameChildren) do
		frameChildren[frameChild]:Hide()
		frameChildren[frameChild]:ClearAllPoints()
		frameChildren[frameChild]:SetParent(nil)
	end

	local i,EventDay,EventIndex
	local _, CurrentMonth, CurrentDay = CalendarGetDate();
	for EventDay = CurrentDay, 31, 1 do
		local i = CalendarGetNumDayEvents(0, EventDay)
		if i > 0 then
			for EventIndex = 1, i, 1 do
				local EventName, EventHour, EventMinute, calendarType, sequenceType, eventType, EventTexture, _, EventInviteStatus = CalendarGetDayEvent(0, EventDay, EventIndex)
				if calendarType == "GUILD_EVENT" then
					local frame,alreadyExists = _CreateEventHeader(EventDay, CurrentMonth)
					frame:Show()
					frame:SetParent(DefileCalendar_Main_ScrollFrame)
					
					frame,alreadyExists = _CreateEventItem(EventIndex, EventDay, EventName, EventHour, EventMinute, EventTexture, EventInviteStatus)
					frame:Show()
					frame:SetParent(DefileCalendar_Main_ScrollFrame)
				end	
			end
		end
	end
	_RedrawDefileFrames()
end

function _CreateEventHeader(EventDay, EventMonth)
	local frameChildren = { DefileCalendar_Main_ScrollFrame:GetChildren() }
	for _, frameChild in ipairs(frameChildren) do
		if frameChild:GetName() == "DefileCalendar_HeaderRow"..EventDay then
			return frameChild, true
		end
	end
	
	frame = CreateFrame("Frame", "DefileCalendar_HeaderRow"..EventDay, DefileCalendar_Main_ScrollFrame, "DefileCalendar_HeaderRow")
	local headerText = CALENDAR_WEEKDAY_NAMES[_GetDayOfWeek(_GetWeekdayIndex(EventDay))].." "..CALENDAR_MONTH_NAMES[EventMonth].." "..EventDay
	frame.HeaderText:SetText(headerText)
	return frame, false
end

function _CreateEventItem(EventIndex, EventDay, EventName, EventHour, EventMinute, EventTexture, EventInviteStatus)
	if EventMinute == 0 then
		EventMinute = "00"
	end
	if EventHour == 0 then
		EventHour = 12 .. ":" .. EventMinute .. " am"
	elseif EventHour > 12 then
		EventHour = EventHour - 12 .. ":" .. EventMinute .. " pm"
	else
		EventHour = EventHour .. ":" .. EventMinute .. " am"
	end
		
	local frameChildren = { DefileCalendar_Main_ScrollFrame:GetChildren() }
	for _, frameChild in ipairs(frameChildren) do
		if frameChild:GetName() == "DefileCalendar_ListRow"..EventDay..","..EventIndex then
			frameChild.EventName:SetText(EventName)
			frameChild.EventTexture:SetTexture("Interface\\LFGFrame\\LFGIcon-"..EventTexture)
			frameChild.EventTime:SetText(EventHour)
			if EventInviteStatus == 7 or EventInviteStatus == 4 then
				frameChild.EventSignup:SetText("Cancel Signup")
			else
				frameChild.EventSignup:SetText("Sign Up")
			end
			return frameChild, true
		end
	end
	frame = CreateFrame("Frame", "DefileCalendar_ListRow"..EventDay..","..EventIndex, DefileCalendar_Main_ScrollFrame, "DefileCalendar_ListRow")
	frame.EventName:SetText(EventName)
	frame.EventTexture:SetTexture("Interface\\LFGFrame\\LFGIcon-"..EventTexture)
	frame.EventTime:SetText(EventHour)
	if EventInviteStatus == 7 or EventInviteStatus == 4 then
		frame.EventSignup:SetText("Cancel Signup")
	else
		frame.EventSignup:SetText("Sign Up")
	end
	frame.EventSignup:RegisterForClicks("AnyUp")
	frame.EventSignup:SetScript("OnClick", function (self, button, down) 
		if self:GetText() == "Sign Up" then
			signupForEvent(EventDay, EventIndex)
			self:SetText("Cancel Signup")
		else
			cancelSignupForEvent(EventDay, EventIndex)
			self:SetText("Sign Up")
		end
	end)

	frame.EventMembers:SetScript("OnEnter", function (...)
		queryDefileEventMembers(EventDay, EventIndex)
	end)
	return frame, false	
end

function _RedrawDefileFrames()
	local frameChildren = { DefileCalendar_Main_ScrollFrame:GetChildren() }
	local numChildren = 0
	for _, frameChild in ipairs(frameChildren) do
		if frameChild:IsShown() then
			frameChild:SetPoint("TOPLEFT", DefileCalendar_Main, "TOPLEFT", 15, (numChildren * -35)-15)
			numChildren = numChildren + 1
		end
	end
	
end

function showDefileCalendar()
	loadDefileEvents()
	DefileCalendar_Main:ClearAllPoints()
	DefileCalendar_Main:SetPoint("TOPLEFT", "CalendarFrame", "TOPRIGHT", 0, -10)
	DefileCalendar_Main:SetPoint("BOTTOMLEFT", "CalendarFrame", "BOTTOMRIGHT", 0, -10)
	DefileCalendar_Main:Show()
end

function hideDefileCalendar()
	DefileCalendar_Main:Hide()
end

function signupForEvent(EventDay, EventIndex)
	CalendarContextEventSignUp(0, EventDay, EventIndex)
end

function cancelSignupForEvent(EventDay, EventIndex)
	CalendarContextInviteRemove(0, EventDay, EventIndex)
end

function queryDefileEventMembers(EventDay, EventIndex)
	CalendarOpenEvent(0, EventDay, EventIndex)
end



DefileCalendar.frame:RegisterEvent("PLAYER_ENTERING_WORLD");
DefileCalendar.frame:RegisterEvent("CALENDAR_UPDATE_EVENT_LIST");
